package org.zeroturnaround.javarebel.integration.servlet.weblogic;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.RebelServletContext;
import org.zeroturnaround.javarebel.integration.generic.BaseServletContextImplClassBytecodeProcessor;
import org.zeroturnaround.javarebel.integration.servlet.JavaRebelResourceServlet;

/**
 * Transforms <code>weblogic.servlet.internal.WebAppServletContext<code>.
 * <p<
 * Also {@link JavaRebelResourceServlet} will be registered as the default servlet.
 * 
 * @author Rein Raudjärv
 */
public class WebAppServletContextCBP extends BaseServletContextImplClassBytecodeProcessor {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    super.process(cp, cl, ctClass);
    
    cp.importPackage("org.zeroturnaround.javarebel");
    
    try {
      CtMethod m = ctClass.getMethod("getResourceAsSource", "(Ljava/lang/String;Z)Lweblogic/utils/classloaders/Source;");
      m.insertBefore(
          "{   " + 
          "      ServletIntegration integration = ServletIntegrationFactory.getInstance();" +
          "      RebelServletContext servletContext = (" + RebelServletContext.class.getName() + ") $0;" +
          "      if (integration.isResourceReplaced(servletContext, $1)) {" +
          "        String path = integration.getRealPath(servletContext, $1);" +
          "        if (path != null) {" +
          "          java.io.File file = new java.io.File(path);" +
          "          if (file.exists())" +
          "            return new weblogic.utils.classloaders.FileSource(file);" +
          "          return null;" +
          "        }" +
          "      }" + 
          "    }");
    } catch (NotFoundException e) {
    }
    
    /*
     * Weblogic 8.x (Tested on 8.1.5)
     * 
     * Register this Servlet Context with JavaRebel and
     * set JavaRebelResourceServlet as the default Servlet
     */
    try {
      CtMethod origMethod = ctClass.getDeclaredMethod("loadDefaultServlet");
      CtMethod copyMethod = CtNewMethod.copy(origMethod, "_loadDefaultServlet", ctClass, null);
      ctClass.addMethod(copyMethod);
      
      origMethod.setBody("{" +
            "  ServletIntegration integration = ServletIntegrationFactory.getInstance();" +
            "  integration.registerServletContext((" + RebelServletContext.class.getName() + ") $0);" +

            "  if (integration.hasReplacedResources((RebelServletContext) $0)) {" +
            "    registerServlet(" +
                    "\"" + JavaRebelResourceServlet.URL_PATTERN + "\", " +
                    "\"" + JavaRebelResourceServlet.class.getName() + "\", " +
                    "new java.util.HashMap());" +
      		"    String str = getDefaultServletName();" +
      		"    if ((str == null) || (str.equals(\"\")))" +
      		"      setDefaultServletName(\"/*\");" +
      		"  } else {" +
      		"    _loadDefaultServlet();" +
      		"  }" +
      		"}");
    } catch (NotFoundException e) {
    }
    
    /*
     * Weblogic 9.x, 10.x (Tested on 9.2.2, 10.3)
     * 
     * Register this Servlet Context with JavaRebel and
     * set JavaRebelResourceServlet as the default Servlet
     */
    try {
      CtMethod origMethod = ctClass.getDeclaredMethod("registerDefaultServlet");
      CtMethod copyMethod = CtNewMethod.copy(origMethod, "_registerDefaultServlet", ctClass, null);
      ctClass.addMethod(copyMethod);
      
      origMethod.setBody("{" +
          "  ServletIntegration integration = ServletIntegrationFactory.getInstance();" +
          "  integration.registerServletContext((" + RebelServletContext.class.getName() + ") $0);" +
          
          "  if (integration.hasReplacedResources((RebelServletContext) $0)) {" +
          "    registerServlet(" +
          "\"" + JavaRebelResourceServlet.SERVLET_NAME + "\", " +
          "\"" + JavaRebelResourceServlet.URL_PATTERN + "\", " +
          "\"" + JavaRebelResourceServlet.class.getName() + "\", " +
          "new java.util.HashMap());" +
          "  } else {" +
          "    _registerDefaultServlet();" +
          "  }" +
      "}");
    } catch (NotFoundException e) {
    }
    
    /*
     * Weblogic 9.x (Tested on 9.2.2)
     * 
     * Unregister this Servlet Context from JavaRebel
     */
    try {
      ctClass.getDeclaredMethod("destroy").insertAfter(
          "{" +
          "  ServletIntegration integration = ServletIntegrationFactory.getInstance();" +
          "  integration.unregisterServletContext((" + RebelServletContext.class.getName() + ") $0);" +
          "}");
    } catch (NotFoundException e) {
    }
    
    /*
     * Weblogic 9.x (Tested on 9.2.2)
     * 
     * Copy new JSP or Tag file into the Weblogic JSP working directory
     */
    try {
      ctClass.getDeclaredMethod("isResourceStale").insertBefore("{" +
              "  " + JspUpdateUtil.class.getName() + ".handleJspUpdate(" +
              "    $0, getRootTempDir(), $1);" +
              "}");
    } catch (NotFoundException e) {
    }
    
  }
}
